@@ -1,11 +1,11 @@
 #!/bin/bash
 ############################################################################
 #
-# Title: Build Package Script for OSX
+# Title: Build Package Script for macOS
 # Authors: CR Oldham, Shane Lee
 # Date: December 2015
 #
-# Description: This creates an OSX package for Salt from the contents of
+# Description: This creates an macOS package for Salt from the contents of
 #              /opt/salt
 #
 # Requirements:
@@ -15,16 +15,27 @@
 #     This script can be passed 2 parameters
 #         $1 : <version> : the version name to give the package (overrides
 #              version of the git repo) (Defaults to the git repo version)
-#         $2 : <package dir> : the staging area for the package defaults to
+#         $2 : <python ver> : the version of python that was built (defaults
+#              to 2)
+#         $3 : <package dir> : the staging area for the package defaults to
 #              /tmp/salt_pkg
 #
 #     Example:
-#         The following will build Salt and stage all files in /tmp/salt_pkg:
+#         The following will build Salt version 2017.7.0 with Python 3 and
+#         stage all files in /tmp/salt_pkg:
 #
-#         ./build.sh
+#         ./build.sh 2017.7.0 3
 #
 ############################################################################
 
+############################################################################
+# Make sure the script is launched with sudo
+############################################################################
+if [[ $(id -u) -ne 0 ]]
+    then
+        exec sudo /bin/bash -c "$(printf '%q ' "$BASH_SOURCE" "$@")"
+fi
+
 ############################################################################
 # Set to Exit on all Errors
 ############################################################################
@@ -45,11 +56,18 @@ else
     VERSION=$1
 fi
 
-# Get/Set temp directory
+# Get/Set Python Version
 if [ "$2" == "" ]; then
+    PYVER=2
+else
+    PYVER=$2
+fi
+
+# Get/Set temp directory
+if [ "$3" == "" ]; then
     PKGDIR=/tmp/salt_pkg
 else
-    PKGDIR=$2
+    PKGDIR=$3
 fi
 
 CPUARCH=`uname -m`
@@ -67,7 +85,7 @@ PKGRESOURCES=$SRCDIR/pkg/osx
 ############################################################################
 if [[ ! -e "$SRCDIR/.git" ]] && [[ ! -e "$SRCDIR/scripts/salt" ]]; then
     echo "This directory doesn't appear to be a git repository."
-    echo "The OS X build process needs some files from a Git checkout of Salt."
+    echo "The macOS build process needs some files from a Git checkout of Salt."
     echo "Run this script from the 'pkg/osx' directory of the Git checkout."
     exit -1
 fi
@@ -86,8 +104,8 @@ mkdir -p $PKGDIR
 ############################################################################
 echo -n -e "\033]0;Build_Pkg: Copy Start Scripts\007"
 
-sudo cp $PKGRESOURCES/scripts/start-*.sh /opt/salt/bin/
-sudo cp $PKGRESOURCES/scripts/salt-config.sh /opt/salt/bin
+cp $PKGRESOURCES/scripts/start-*.sh /opt/salt/bin/
+cp $PKGRESOURCES/scripts/salt-config.sh /opt/salt/bin
 
 ############################################################################
 # Copy Service Definitions from Salt Repo to the Package Directory
@@ -108,16 +126,20 @@ cp $PKGRESOURCES/scripts/com.saltstack.salt.api.plist $PKGDIR/Library/LaunchDaem
 ############################################################################
 echo -n -e "\033]0;Build_Pkg: Trim unneeded files\007"
 
-sudo rm -rdf $PKGDIR/opt/salt/bin/pkg-config
-sudo rm -rdf $PKGDIR/opt/salt/lib/pkgconfig
-sudo rm -rdf $PKGDIR/opt/salt/lib/engines
-sudo rm -rdf $PKGDIR/opt/salt/share/aclocal
-sudo rm -rdf $PKGDIR/opt/salt/share/doc
-sudo rm -rdf $PKGDIR/opt/salt/share/man/man1/pkg-config.1
-sudo rm -rdf $PKGDIR/opt/salt/lib/python2.7/test
+rm -rdf $PKGDIR/opt/salt/bin/pkg-config
+rm -rdf $PKGDIR/opt/salt/lib/pkgconfig
+rm -rdf $PKGDIR/opt/salt/lib/engines
+rm -rdf $PKGDIR/opt/salt/share/aclocal
+rm -rdf $PKGDIR/opt/salt/share/doc
+rm -rdf $PKGDIR/opt/salt/share/man/man1/pkg-config.1
+if [ "$PYVER" == "2" ]; then
+    rm -rdf $PKGDIR/opt/salt/lib/python2.7/test
+else
+    rm -rdf $PKGDIR/opt/salt/lib/python3.5/test
+fi
 
 echo -n -e "\033]0;Build_Pkg: Remove compiled python files\007"
-sudo find $PKGDIR/opt/salt -name '*.pyc' -type f -delete
+find $PKGDIR/opt/salt -name '*.pyc' -type f -delete
 
 ############################################################################
 # Copy Config Files from Salt Repo to the Package Directory
@@ -133,15 +155,30 @@ cp $SRCDIR/conf/master $PKGDIR/etc/salt/master.dist
 ############################################################################
 echo -n -e "\033]0;Build_Pkg: Add Version to .xml\007"
 
+if [ "$PYVER" == "2" ]; then
+    TITLE="Salt $VERSION"
+    DESC="Salt $VERSION with Python 2"
+else
+    TITLE="Salt $VERSION (Python 3)"
+    DESC="Salt $VERSION with Python 3"
+fi
+
 cd $PKGRESOURCES
 cp distribution.xml.dist distribution.xml
-SEDSTR="s/@VERSION@/$VERSION/"
-echo $SEDSTR
-sed -i '' $SEDSTR distribution.xml
+SEDSTR="s/@TITLE@/$TITLE/g"
+sed -E -i '' "$SEDSTR" distribution.xml
 
-SEDSTR="s/@CPUARCH@/$CPUARCH/"
-echo $SEDSTR
-sed -i '' $SEDSTR distribution.xml
+SEDSTR="s/@DESC@/$DESC/g"
+sed -E -i '' "$SEDSTR" distribution.xml
+
+SEDSTR="s/@VERSION@/$VERSION/g"
+sed -E -i '' "$SEDSTR" distribution.xml
+
+SEDSTR="s/@PYVER@/$PYVER/g"
+sed -E -i '' "$SEDSTR" distribution.xml
+
+SEDSTR="s/@CPUARCH@/$CPUARCH/g"
+sed -i '' "$SEDSTR" distribution.xml
 
 ############################################################################
 # Build the Package
@@ -152,10 +189,10 @@ pkgbuild --root=$PKGDIR \
          --scripts=pkg-scripts \
          --identifier=com.saltstack.salt \
          --version=$VERSION \
-         --ownership=recommended salt-src-$VERSION-$CPUARCH.pkg
+         --ownership=recommended salt-src-$VERSION-py$PYVER-$CPUARCH.pkg
 
 productbuild --resources=pkg-resources \
              --distribution=distribution.xml  \
-             --package-path=salt-src-$VERSION-$CPUARCH.pkg \
-             --version=$VERSION salt-$VERSION-$CPUARCH.pkg
+             --package-path=salt-src-$VERSION-py$PYVER-$CPUARCH.pkg \
+             --version=$VERSION salt-$VERSION-py$PYVER-$CPUARCH.pkg
 
