@@ -41,6 +41,20 @@ or clusters are available.
     as this makes all master configuration settings available in all minion's
     pillars.
 
+Etcd profile configuration can be overridden using following arguments: ``host``,
+``port``, ``username``, ``password``, ``ca``, ``client_key`` and ``client_cert``.
+
+.. code-block:: yaml
+
+    my-value:
+      etcd.set:
+        - name: /path/to/key
+        - value: value
+        - host: 127.0.0.1
+        - port: 2379
+        - username: user
+        - password: pass
+
 Available Functions
 -------------------
 
@@ -104,8 +118,7 @@ Available Functions
 '''
 
 # Import Python Libs
-from __future__ import absolute_import
-
+from __future__ import absolute_import, print_function, unicode_literals
 
 # Define the module's virtual name
 __virtualname__ = 'etcd'
@@ -118,7 +131,7 @@ __func_alias__ = {
 
 # Import third party libs
 try:
-    import etcd
+    import salt.utils.etcd_util  # pylint: disable=W0611
     HAS_ETCD = True
 except ImportError:
     HAS_ETCD = False
@@ -132,7 +145,7 @@ def __virtual__():
     return __virtualname__ if HAS_ETCD else False
 
 
-def set_(name, value, profile=None):
+def set_(name, value, profile=None, **kwargs):
     '''
     Set a key in etcd and can be called as ``set``.
 
@@ -143,6 +156,13 @@ def set_(name, value, profile=None):
     profile
         Optional, defaults to ``None``. Sets the etcd profile to use which has
         been defined in the Salt Master config.
+
+        .. code-block:: yaml
+
+            my_etd_config:
+              etcd.host: 127.0.0.1
+              etcd.port: 4001
+
     '''
 
     created = False
@@ -154,15 +174,13 @@ def set_(name, value, profile=None):
         'changes': {}
     }
 
-    try:
-        current = __salt__['etcd.get'](name, profile=profile)
-    except etcd.EtcdKeyNotFound:
+    current = __salt__['etcd.get'](name, profile=profile, **kwargs)
+    if not current:
         created = True
-        current = None
 
-    result = __salt__['etcd.set'](name, value, profile=profile)
+    result = __salt__['etcd.set'](name, value, profile=profile, **kwargs)
 
-    if result != current:
+    if result and result != current:
         if created:
             rtn['comment'] = 'New key created'
         else:
@@ -174,7 +192,7 @@ def set_(name, value, profile=None):
     return rtn
 
 
-def wait_set(name, value, profile=None):
+def wait_set(name, value, profile=None, **kwargs):
     '''
     Set a key in etcd only if the watch statement calls it. This function is
     also aliased as ``wait_set``.
@@ -186,6 +204,13 @@ def wait_set(name, value, profile=None):
     profile
         The etcd profile to use that has been configured on the Salt Master,
         this is optional and defaults to ``None``.
+
+        .. code-block:: yaml
+
+            my_etd_config:
+              etcd.host: 127.0.0.1
+              etcd.port: 4001
+
     '''
 
     return {
@@ -196,7 +221,49 @@ def wait_set(name, value, profile=None):
     }
 
 
-def rm_(name, recurse=False, profile=None):
+def directory(name, profile=None, **kwargs):
+    '''
+    Create a directory in etcd.
+
+    name
+        The etcd directory name, for example: ``/foo/bar/baz``.
+    profile
+        Optional, defaults to ``None``. Sets the etcd profile to use which has
+        been defined in the Salt Master config.
+
+        .. code-block:: yaml
+
+            my_etd_config:
+              etcd.host: 127.0.0.1
+              etcd.port: 4001
+    '''
+
+    created = False
+
+    rtn = {
+        'name': name,
+        'comment': 'Directory exists',
+        'result': True,
+        'changes': {}
+    }
+
+    current = __salt__['etcd.get'](name, profile=profile, recurse=True, **kwargs)
+    if not current:
+        created = True
+
+    result = __salt__['etcd.set'](name, None, directory=True, profile=profile, **kwargs)
+
+    if result and result != current:
+        if created:
+            rtn['comment'] = 'New directory created'
+            rtn['changes'] = {
+                name: 'Created'
+            }
+
+    return rtn
+
+
+def rm_(name, recurse=False, profile=None, **kwargs):
     '''
     Deletes a key from etcd. This function is also aliased as ``rm``.
 
@@ -207,6 +274,12 @@ def rm_(name, recurse=False, profile=None):
     profile
         Optional, defaults to ``None``. Sets the etcd profile to use which has
         been defined in the Salt Master config.
+
+        .. code-block:: yaml
+
+            my_etd_config:
+              etcd.host: 127.0.0.1
+              etcd.port: 4001
     '''
 
     rtn = {
@@ -215,13 +288,11 @@ def rm_(name, recurse=False, profile=None):
         'changes': {}
     }
 
-    try:
-        __salt__['etcd.get'](name, profile=profile)
-    except etcd.EtcdKeyNotFound:
+    if not __salt__['etcd.get'](name, profile=profile, **kwargs):
         rtn['comment'] = 'Key does not exist'
         return rtn
 
-    if __salt__['etcd.rm'](name, recurse=recurse, profile=profile):
+    if __salt__['etcd.rm'](name, recurse=recurse, profile=profile, **kwargs):
         rtn['comment'] = 'Key removed'
         rtn['changes'] = {
             name: 'Deleted'
@@ -232,7 +303,7 @@ def rm_(name, recurse=False, profile=None):
     return rtn
 
 
-def wait_rm(name, recurse=False, profile=None):
+def wait_rm(name, recurse=False, profile=None, **kwargs):
     '''
     Deletes a key from etcd only if the watch statement calls it.
     This function is also aliased as ``wait_rm``.
@@ -241,10 +312,16 @@ def wait_rm(name, recurse=False, profile=None):
         The etcd key name to remove, for example ``/foo/bar/baz``.
     recurse
         Optional, defaults to ``False``. If ``True`` performs a recursive
-        delete, see: https://python-etcd.readthedocs.org/en/latest/#delete-a-key.
+        delete, see: https://python-etcd.readthedocs.io/en/latest/#delete-a-key.
     profile
         Optional, defaults to ``None``. Sets the etcd profile to use which has
         been defined in the Salt Master config.
+
+        .. code-block:: yaml
+
+            my_etd_config:
+              etcd.host: 127.0.0.1
+              etcd.port: 4001
     '''
 
     return {
