@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 
 # Import python libs
-from __future__ import absolute_import
+from __future__ import absolute_import, print_function, unicode_literals
 import logging
 
 # Import Salt Testing libs
@@ -11,11 +11,11 @@ from tests.support.helpers import destructiveTest
 from tests.support.mixins import SaltReturnAssertsMixin
 
 # Import salt libs
-import salt.utils
+import salt.utils.path
 from salt.modules import mysql as mysqlmod
 
 # Import 3rd-party libs
-import salt.ext.six as six
+from salt.ext import six
 from salt.ext.six.moves import range  # pylint: disable=import-error,redefined-builtin
 
 log = logging.getLogger(__name__)
@@ -26,7 +26,7 @@ try:
 except Exception:
     NO_MYSQL = True
 
-if not salt.utils.which('mysqladmin'):
+if not salt.utils.path.which('mysqladmin'):
     NO_MYSQL = True
 
 
@@ -413,11 +413,11 @@ class MysqlModuleDbTest(ModuleCase, SaltReturnAssertsMixin):
                     tblname=mysqlmod.quote_identifier(tablename)
             ))
             for x in range(100):
-                insert_query += "('foo"+str(x)+"'),"
+                insert_query += "('foo"+six.text_type(x)+"'),"
             insert_query += "('bar');"
 
             # populate database
-            log.info('Adding table \'{0}\''.format(tablename,))
+            log.info('Adding table \'%s\'', tablename)
             ret = self.run_function(
               'mysql.query',
               database=dbname,
@@ -434,7 +434,7 @@ class MysqlModuleDbTest(ModuleCase, SaltReturnAssertsMixin):
                     )
                 )
             self.assertEqual(ret['rows affected'], 0)
-            log.info('Populating table \'{0}\''.format(tablename,))
+            log.info('Populating table \'%s\'', tablename)
             ret = self.run_function(
               'mysql.query',
               database=dbname,
@@ -451,7 +451,7 @@ class MysqlModuleDbTest(ModuleCase, SaltReturnAssertsMixin):
                     )
                 )
             self.assertEqual(ret['rows affected'], 101)
-            log.info('Removing some rows on table\'{0}\''.format(tablename,))
+            log.info('Removing some rows on table\'%s\'', tablename)
             ret = self.run_function(
               'mysql.query',
               database=dbname,
@@ -1280,6 +1280,7 @@ class MysqlModuleUserGrantTest(ModuleCase, SaltReturnAssertsMixin):
     testdb1 = 'tes.t\'"saltdb'
     testdb2 = 't_st `(:=salt%b)'
     testdb3 = 'test `(:=salteeb)'
+    test_file_query_db = 'test_query'
     table1 = 'foo'
     table2 = "foo `\'%_bar"
     users = {
@@ -1356,7 +1357,7 @@ class MysqlModuleUserGrantTest(ModuleCase, SaltReturnAssertsMixin):
             tblname=mysqlmod.quote_identifier(self.table1),
             engine='MYISAM',
         ))
-        log.info('Adding table \'{0}\''.format(self.table1,))
+        log.info('Adding table \'%s\'', self.table1)
         self.run_function(
             'mysql.query',
             database=self.testdb2,
@@ -1370,7 +1371,7 @@ class MysqlModuleUserGrantTest(ModuleCase, SaltReturnAssertsMixin):
             tblname=mysqlmod.quote_identifier(self.table2),
             engine='MYISAM',
         ))
-        log.info('Adding table \'{0}\''.format(self.table2,))
+        log.info('Adding table \'%s\'', self.table2)
         self.run_function(
             'mysql.query',
             database=self.testdb2,
@@ -1391,13 +1392,19 @@ class MysqlModuleUserGrantTest(ModuleCase, SaltReturnAssertsMixin):
             name=self.testdb1,
             connection_user=self.user,
             connection_pass=self.password,
-       )
+        )
         self.run_function(
             'mysql.db_remove',
             name=self.testdb2,
             connection_user=self.user,
             connection_pass=self.password,
-       )
+        )
+        self.run_function(
+            'mysql.db_remove',
+            name=self.test_file_query_db,
+            connection_user=self.user,
+            connection_pass=self.password,
+        )
 
     def _userCreation(self,
                       uname,
@@ -1627,3 +1634,123 @@ class MysqlModuleUserGrantTest(ModuleCase, SaltReturnAssertsMixin):
             "GRANT USAGE ON *.* TO ''@'localhost'",
             "GRANT DELETE ON `test ``(:=salteeb)`.* TO ''@'localhost'"
         ])
+
+
+@skipIf(
+    NO_MYSQL,
+    'Please install MySQL bindings and a MySQL Server before running'
+    'MySQL integration tests.'
+)
+class MysqlModuleFileQueryTest(ModuleCase, SaltReturnAssertsMixin):
+    '''
+    Test file query module
+    '''
+
+    user = 'root'
+    password = 'poney'
+    testdb = 'test_file_query'
+
+    @destructiveTest
+    def setUp(self):
+        '''
+        Test presence of MySQL server, enforce a root password, create users
+        '''
+        super(MysqlModuleFileQueryTest, self).setUp()
+        NO_MYSQL_SERVER = True
+        # now ensure we know the mysql root password
+        # one of theses two at least should work
+        ret1 = self.run_state(
+            'cmd.run',
+             name='mysqladmin --host="localhost" -u '
+               + self.user
+               + ' flush-privileges password "'
+               + self.password
+               + '"'
+        )
+        ret2 = self.run_state(
+            'cmd.run',
+             name='mysqladmin --host="localhost" -u '
+               + self.user
+               + ' --password="'
+               + self.password
+               + '" flush-privileges password "'
+               + self.password
+               + '"'
+        )
+        key, value = ret2.popitem()
+        if value['result']:
+            NO_MYSQL_SERVER = False
+        else:
+            self.skipTest('No MySQL Server running, or no root access on it.')
+        # Create some users and a test db
+        self.run_function(
+            'mysql.db_create',
+            name=self.testdb,
+            connection_user=self.user,
+            connection_pass=self.password,
+            connection_db='mysql',
+        )
+
+    @destructiveTest
+    def tearDown(self):
+        '''
+        Removes created users and db
+        '''
+        self.run_function(
+            'mysql.db_remove',
+            name=self.testdb,
+            connection_user=self.user,
+            connection_pass=self.password,
+            connection_db='mysql',
+       )
+
+    @destructiveTest
+    def test_update_file_query(self):
+        '''
+        Test query without any output
+        '''
+        ret = self.run_function(
+          'mysql.file_query',
+          database=self.testdb,
+          file_name='salt://mysql/update_query.sql',
+          character_set='utf8',
+          collate='utf8_general_ci',
+          connection_user=self.user,
+          connection_pass=self.password
+        )
+        self.assertTrue('query time' in ret)
+        ret.pop('query time')
+        self.assertEqual(ret, {'rows affected': 2})
+
+    @destructiveTest
+    def test_select_file_query(self):
+        '''
+        Test query with table output
+        '''
+        ret = self.run_function(
+          'mysql.file_query',
+          database=self.testdb,
+          file_name='salt://mysql/select_query.sql',
+          character_set='utf8',
+          collate='utf8_general_ci',
+          connection_user=self.user,
+          connection_pass=self.password
+        )
+        expected = {
+            'rows affected': 5,
+            'rows returned': 4,
+            'results': [
+                [
+                    ['2'],
+                    ['3'],
+                    ['4'],
+                    ['5']
+                ]
+            ],
+            'columns': [
+                ['a']
+            ],
+        }
+        self.assertTrue('query time' in ret)
+        ret.pop('query time')
+        self.assertEqual(ret, expected)
