@@ -11,10 +11,12 @@ import textwrap
 
 # Import Salt Testing libs
 from tests.support.case import ModuleCase
+from tests.support.helpers import flaky
 from tests.support.paths import TMP_PILLAR_TREE
 
 # Import Salt Libs
 import salt.utils.files
+import salt.utils.stringutils
 
 
 class SaltUtilModuleTest(ModuleCase):
@@ -172,6 +174,7 @@ class SaltUtilSyncPillarTest(ModuleCase):
     Testcase for the saltutil sync pillar module
     '''
 
+    @flaky
     def test_pillar_refresh(self):
         '''
         test pillar refresh module
@@ -194,7 +197,18 @@ class SaltUtilSyncPillarTest(ModuleCase):
                      '''))
 
         self.run_function('saltutil.refresh_pillar')
-        self.run_function('test.sleep', [5])
+
+        pillar = False
+        timeout = time.time() + 30
+        while not pillar:
+            post_pillar = self.run_function('pillar.raw')
+            try:
+                self.assertIn(pillar_key, post_pillar.get(pillar_key, 'didnotwork'))
+                pillar = True
+            except AssertionError:
+                if time.time() > timeout:
+                    self.assertIn(pillar_key, post_pillar.get(pillar_key, 'didnotwork'))
+                continue
 
         post_pillar = self.run_function('pillar.raw')
         self.assertIn(pillar_key, post_pillar.get(pillar_key, 'didnotwork'))
