@@ -3,8 +3,8 @@
 '''
 tests for pkg state
 '''
-# Import python libs
-from __future__ import absolute_import
+# Import Python libs
+from __future__ import absolute_import, print_function, unicode_literals
 import logging
 import os
 import time
@@ -16,13 +16,17 @@ from tests.support.unit import skipIf
 from tests.support.helpers import (
     destructiveTest,
     requires_system_grains,
-    requires_salt_modules
+    requires_salt_modules,
+    flaky
 )
 
-# Import salt libs
-import salt.utils
+# Import Salt libs
+import salt.utils.path
+import salt.utils.pkg.rpm
+import salt.utils.platform
 
 # Import 3rd-party libs
+from salt.ext import six
 from salt.ext.six.moves import range  # pylint: disable=import-error,redefined-builtin
 
 log = logging.getLogger(__name__)
@@ -34,11 +38,15 @@ _PKG_TARGETS = {
     'Debian': ['python-plist', 'apg'],
     'RedHat': ['units', 'zsh-html'],
     'FreeBSD': ['aalib', 'pth'],
-    'Suse': ['aalib', 'python-pssh'],
+    'Suse': ['aalib', 'rpm-python'],
     'MacOS': ['libpng', 'jpeg'],
     'Windows': ['putty', '7zip'],
 }
 
+_PKG_CAP_TARGETS = {
+    'Suse': [('w3m_ssl', 'w3m')],
+}
+
 _PKG_TARGETS_32 = {
     'CentOS': 'xz-devel.i686'
 }
@@ -99,7 +107,7 @@ def pkgmgr_avail(run_function, grains):
         Return True if any locks are found for path
         '''
         # Try lsof if it's available
-        if salt.utils.which('lsof'):
+        if salt.utils.path.which('lsof'):
             lock = run_function('cmd.run', ['lsof {0}'.format(path)])
             return True if len(lock) else False
 
@@ -143,6 +151,7 @@ def latest_version(run_function, *names):
     return ret
 
 
+@flaky
 @destructiveTest
 @requires_salt_modules('pkg.version', 'pkg.latest_version')
 class PkgTest(ModuleCase, SaltReturnAssertsMixin):
@@ -467,7 +476,7 @@ class PkgTest(ModuleCase, SaltReturnAssertsMixin):
         ret = self.run_state('pkg.removed', name=target)
         self.assertSaltTrueReturn(ret)
 
-    @skipIf(salt.utils.is_windows(), 'minion is windows')
+    @skipIf(salt.utils.platform.is_windows(), 'minion is windows')
     def test_pkg_009_latest_with_epoch(self):
         '''
         This tests for the following issue:
@@ -501,7 +510,7 @@ class PkgTest(ModuleCase, SaltReturnAssertsMixin):
         pkgquery = 'version'
 
         ret = self.run_function('pkg.info_installed', [package])
-        self.assertTrue(pkgquery in str(ret))
+        self.assertTrue(pkgquery in six.text_type(ret))
 
     @requires_system_grains
     def test_pkg_011_latest(self, grains=None):
@@ -670,6 +679,55 @@ class PkgTest(ModuleCase, SaltReturnAssertsMixin):
         self.assertSaltTrueReturn(ret)
 
     @requires_system_grains
+    def test_pkg_014_installed_with_comparison_operator(self, grains=None):
+        '''
+        This is a destructive test as it installs and then removes a package
+        '''
+        # Skip test if package manager not available
+        if not pkgmgr_avail(self.run_function, self.run_function('grains.items')):
+            self.skipTest('Package manager is not available')
+
+        os_family = grains.get('os_family', '')
+        if os_family not in ('Debian', 'RedHat'):
+            self.skipTest('Comparison operator not specially implemented')
+
+        pkg_targets = _PKG_TARGETS.get(os_family, [])
+
+        # Make sure that we have targets that match the os_family. If this
+        # fails then the _PKG_TARGETS dict above needs to have an entry added,
+        # with two packages that are not installed before these tests are run
+        self.assertTrue(pkg_targets)
+
+        target = pkg_targets[0]
+        version = self.run_function('pkg.version', [target])
+
+        # If this assert fails, we need to find new targets, this test needs to
+        # be able to test successful installation of packages, so this package
+        # needs to not be installed before we run the states below
+        self.assertFalse(version)
+
+        latest_version = self.run_function(
+            'pkg.latest_version',
+            [target],
+            refresh=False)
+
+        try:
+            ret = self.run_state(
+                'pkg.installed',
+                name=target,
+                version='<9999999',
+                refresh=False,
+            )
+            self.assertSaltTrueReturn(ret)
+
+            # The version that was installed should be the latest available
+            version = self.run_function('pkg.version', [target])
+            self.assertTrue(version, latest_version)
+        finally:
+            # Clean up
+            ret = self.run_state('pkg.removed', name=target)
+            self.assertSaltTrueReturn(ret)
+
     def test_pkg_014_installed_missing_release(self, grains=None):  # pylint: disable=unused-argument
         '''
         Tests that a version number missing the release portion still resolves
@@ -698,7 +756,7 @@ class PkgTest(ModuleCase, SaltReturnAssertsMixin):
         ret = self.run_state(
             'pkg.installed',
             name=target,
-            version=salt.utils.str_version_to_evr(version)[1],
+            version=salt.utils.pkg.rpm.version_to_evr(version)[1],
             refresh=False,
         )
         self.assertSaltTrueReturn(ret)
@@ -737,6 +795,263 @@ class PkgTest(ModuleCase, SaltReturnAssertsMixin):
                                       '\'handle_missing_pkg_group\': Group \'handle_missing_pkg_group\' '
                                       'not found.')
 
+    @skipIf(salt.utils.platform.is_windows(), 'minion is windows')
+    @requires_system_grains
+    def test_pkg_cap_001_installed(self, grains=None):
+        '''
+        This is a destructive test as it installs and then removes a package
+        '''
+        # Skip test if package manager not available
+        if not pkgmgr_avail(self.run_function, self.run_function('grains.items')):
+            self.skipTest('Package manager is not available')
+
+        os_family = grains.get('os_family', '')
+        pkg_cap_targets = _PKG_CAP_TARGETS.get(os_family, [])
+        if not len(pkg_cap_targets) > 0:
+            self.skipTest('Capability not provided')
+
+        target, realpkg = pkg_cap_targets[0]
+        version = self.run_function('pkg.version', [target])
+        realver = self.run_function('pkg.version', [realpkg])
+
+        # If this assert fails, we need to find new targets, this test needs to
+        # be able to test successful installation of packages, so this package
+        # needs to not be installed before we run the states below
+        self.assertFalse(version)
+        self.assertFalse(realver)
+
+        ret = self.run_state('pkg.installed', name=target, refresh=False, resolve_capabilities=True, test=True)
+        self.assertInSaltComment("The following packages would be installed/updated: {0}".format(realpkg), ret)
+        ret = self.run_state('pkg.installed', name=target, refresh=False, resolve_capabilities=True)
+        self.assertSaltTrueReturn(ret)
+        ret = self.run_state('pkg.removed', name=realpkg)
+        self.assertSaltTrueReturn(ret)
+
+    @skipIf(salt.utils.platform.is_windows(), 'minion is windows')
+    @requires_system_grains
+    def test_pkg_cap_002_already_installed(self, grains=None):
+        '''
+        This is a destructive test as it installs and then removes a package
+        '''
+        # Skip test if package manager not available
+        if not pkgmgr_avail(self.run_function, self.run_function('grains.items')):
+            self.skipTest('Package manager is not available')
+
+        os_family = grains.get('os_family', '')
+        pkg_cap_targets = _PKG_CAP_TARGETS.get(os_family, [])
+        if not len(pkg_cap_targets) > 0:
+            self.skipTest('Capability not provided')
+
+        target, realpkg = pkg_cap_targets[0]
+        version = self.run_function('pkg.version', [target])
+        realver = self.run_function('pkg.version', [realpkg])
+
+        # If this assert fails, we need to find new targets, this test needs to
+        # be able to test successful installation of packages, so this package
+        # needs to not be installed before we run the states below
+        self.assertFalse(version)
+        self.assertFalse(realver)
+
+        # install the package already
+        ret = self.run_state('pkg.installed', name=realpkg, refresh=False)
+
+        ret = self.run_state('pkg.installed', name=target, refresh=False, resolve_capabilities=True, test=True)
+        self.assertInSaltComment("All specified packages are already installed", ret)
+
+        ret = self.run_state('pkg.installed', name=target, refresh=False, resolve_capabilities=True)
+        self.assertSaltTrueReturn(ret)
+
+        self.assertInSaltComment("packages are already installed", ret)
+        ret = self.run_state('pkg.removed', name=realpkg)
+        self.assertSaltTrueReturn(ret)
+
+    @skipIf(salt.utils.platform.is_windows(), 'minion is windows')
+    @requires_system_grains
+    def test_pkg_cap_003_installed_multipkg_with_version(self, grains=None):
+        '''
+        This is a destructive test as it installs and then removes two packages
+        '''
+        # Skip test if package manager not available
+        if not pkgmgr_avail(self.run_function, self.run_function('grains.items')):
+            self.skipTest('Package manager is not available')
+
+        os_family = grains.get('os_family', '')
+        pkg_cap_targets = _PKG_CAP_TARGETS.get(os_family, [])
+        if not len(pkg_cap_targets) > 0:
+            self.skipTest('Capability not provided')
+        pkg_targets = _PKG_TARGETS.get(os_family, [])
+
+        # Don't perform this test on FreeBSD since version specification is not
+        # supported.
+        if os_family == 'FreeBSD':
+            return
+
+        # Make sure that we have targets that match the os_family. If this
+        # fails then the _PKG_TARGETS dict above needs to have an entry added,
+        # with two packages that are not installed before these tests are run
+        self.assertTrue(bool(pkg_cap_targets))
+        self.assertTrue(bool(pkg_targets))
+
+        if os_family == 'Arch':
+            for idx in range(13):
+                if idx == 12:
+                    raise Exception('Package database locked after 60 seconds, '
+                                    'bailing out')
+                if not os.path.isfile('/var/lib/pacman/db.lck'):
+                    break
+                time.sleep(5)
+
+        capability, realpkg = pkg_cap_targets[0]
+        version = latest_version(self.run_function, pkg_targets[0])
+        realver = latest_version(self.run_function, realpkg)
+
+        # If this assert fails, we need to find new targets, this test needs to
+        # be able to test successful installation of packages, so these
+        # packages need to not be installed before we run the states below
+        self.assertTrue(bool(version))
+        self.assertTrue(bool(realver))
+
+        pkgs = [{pkg_targets[0]: version}, pkg_targets[1], {capability: realver}]
+        ret = self.run_state('pkg.installed',
+                             name='test_pkg_cap_003_installed_multipkg_with_version-install',
+                             pkgs=pkgs,
+                             refresh=False)
+        self.assertSaltFalseReturn(ret)
+
+        ret = self.run_state('pkg.installed',
+                             name='test_pkg_cap_003_installed_multipkg_with_version-install-capability',
+                             pkgs=pkgs,
+                             refresh=False, resolve_capabilities=True, test=True)
+        self.assertInSaltComment("packages would be installed/updated", ret)
+        self.assertInSaltComment("{0}={1}".format(realpkg, realver), ret)
+
+        ret = self.run_state('pkg.installed',
+                             name='test_pkg_cap_003_installed_multipkg_with_version-install-capability',
+                             pkgs=pkgs,
+                             refresh=False, resolve_capabilities=True)
+        self.assertSaltTrueReturn(ret)
+        cleanup_pkgs = pkg_targets
+        cleanup_pkgs.append(realpkg)
+        ret = self.run_state('pkg.removed',
+                             name='test_pkg_cap_003_installed_multipkg_with_version-remove',
+                             pkgs=cleanup_pkgs)
+        self.assertSaltTrueReturn(ret)
+
+    @skipIf(salt.utils.platform.is_windows(), 'minion is windows')
+    @requires_system_grains
+    def test_pkg_cap_004_latest(self, grains=None):
+        '''
+        This tests pkg.latest with a package that has no epoch (or a zero
+        epoch).
+        '''
+        # Skip test if package manager not available
+        if not pkgmgr_avail(self.run_function, self.run_function('grains.items')):
+            self.skipTest('Package manager is not available')
+
+        os_family = grains.get('os_family', '')
+        pkg_cap_targets = _PKG_CAP_TARGETS.get(os_family, [])
+        if not len(pkg_cap_targets) > 0:
+            self.skipTest('Capability not provided')
+
+        target, realpkg = pkg_cap_targets[0]
+        version = self.run_function('pkg.version', [target])
+        realver = self.run_function('pkg.version', [realpkg])
+
+        # If this assert fails, we need to find new targets, this test needs to
+        # be able to test successful installation of packages, so this package
+        # needs to not be installed before we run the states below
+        self.assertFalse(version)
+        self.assertFalse(realver)
+
+        ret = self.run_state('pkg.latest', name=target, refresh=False, resolve_capabilities=True, test=True)
+        self.assertInSaltComment("The following packages would be installed/upgraded: {0}".format(realpkg), ret)
+        ret = self.run_state('pkg.latest', name=target, refresh=False, resolve_capabilities=True)
+        self.assertSaltTrueReturn(ret)
+
+        ret = self.run_state('pkg.latest', name=target, refresh=False, resolve_capabilities=True)
+        self.assertSaltTrueReturn(ret)
+        self.assertInSaltComment("is already up-to-date", ret)
+
+        ret = self.run_state('pkg.removed', name=realpkg)
+        self.assertSaltTrueReturn(ret)
+
+    @skipIf(salt.utils.platform.is_windows(), 'minion is windows')
+    @requires_system_grains
+    def test_pkg_cap_005_downloaded(self, grains=None):
+        '''
+        This is a destructive test as it installs and then removes a package
+        '''
+        # Skip test if package manager not available
+        if not pkgmgr_avail(self.run_function, self.run_function('grains.items')):
+            self.skipTest('Package manager is not available')
+
+        os_family = grains.get('os_family', '')
+        pkg_cap_targets = _PKG_CAP_TARGETS.get(os_family, [])
+        if not len(pkg_cap_targets) > 0:
+            self.skipTest('Capability not provided')
+
+        target, realpkg = pkg_cap_targets[0]
+        version = self.run_function('pkg.version', [target])
+        realver = self.run_function('pkg.version', [realpkg])
+
+        # If this assert fails, we need to find new targets, this test needs to
+        # be able to test successful installation of packages, so this package
+        # needs to not be installed before we run the states below
+        self.assertFalse(version)
+        self.assertFalse(realver)
+
+        ret = self.run_state('pkg.downloaded', name=target, refresh=False)
+        self.assertSaltFalseReturn(ret)
+
+        ret = self.run_state('pkg.downloaded', name=target, refresh=False, resolve_capabilities=True, test=True)
+        self.assertInSaltComment("The following packages would be downloaded: {0}".format(realpkg), ret)
+
+        ret = self.run_state('pkg.downloaded', name=target, refresh=False, resolve_capabilities=True)
+        self.assertSaltTrueReturn(ret)
+
+    @skipIf(salt.utils.platform.is_windows(), 'minion is windows')
+    @requires_system_grains
+    def test_pkg_cap_006_uptodate(self, grains=None):
+        '''
+        This is a destructive test as it installs and then removes a package
+        '''
+        # Skip test if package manager not available
+        if not pkgmgr_avail(self.run_function, self.run_function('grains.items')):
+            self.skipTest('Package manager is not available')
+
+        os_family = grains.get('os_family', '')
+        pkg_cap_targets = _PKG_CAP_TARGETS.get(os_family, [])
+        if not len(pkg_cap_targets) > 0:
+            self.skipTest('Capability not provided')
+
+        target, realpkg = pkg_cap_targets[0]
+        version = self.run_function('pkg.version', [target])
+        realver = self.run_function('pkg.version', [realpkg])
+
+        # If this assert fails, we need to find new targets, this test needs to
+        # be able to test successful installation of packages, so this package
+        # needs to not be installed before we run the states below
+        self.assertFalse(version)
+        self.assertFalse(realver)
+
+        ret = self.run_state('pkg.installed', name=target,
+                             refresh=False, resolve_capabilities=True)
+        self.assertSaltTrueReturn(ret)
+        ret = self.run_state('pkg.uptodate',
+                             name='test_pkg_cap_006_uptodate',
+                             pkgs=[target],
+                             refresh=False,
+                             resolve_capabilities=True)
+        self.assertSaltTrueReturn(ret)
+        self.assertInSaltComment("System is already up-to-date", ret)
+        ret = self.run_state('pkg.removed', name=realpkg)
+        self.assertSaltTrueReturn(ret)
+        ret = self.run_state('pkg.uptodate',
+                             name='test_pkg_cap_006_uptodate',
+                             refresh=False,
+                             test=True)
+        self.assertInSaltComment("System update will be performed", ret)
+
     @requires_salt_modules('pkg.hold', 'pkg.unhold')
     @requires_system_grains
     def test_pkg_015_installed_held(self, grains=None):  # pylint: disable=unused-argument
