@@ -58,9 +58,9 @@ except ImportError as exc:
     raise exc
 
 from tests.integration import TestDaemon  # pylint: disable=W0403
-import salt.utils
+import salt.utils.platform
 
-if not salt.utils.is_windows():
+if not salt.utils.platform.is_windows():
     import resource
 
 # Import Salt Testing libs
@@ -101,6 +101,9 @@ TEST_SUITES = {
     'unit':
        {'display_name': 'Unit',
         'path': 'unit'},
+    'kitchen':
+       {'display_name': 'Kitchen',
+        'path': 'kitchen'},
     'module':
        {'display_name': 'Module',
         'path': 'integration/modules'},
@@ -170,6 +173,12 @@ TEST_SUITES = {
     'external_api':
         {'display_name': 'ExternalAPIs',
          'path': 'integration/externalapi'},
+    'daemons':
+        {'display_name': 'Daemon',
+         'path': 'integration/daemons'},
+    'scheduler':
+        {'display_name': 'Scheduler',
+         'path': 'integration/scheduler'},
 }
 
 
@@ -179,7 +188,7 @@ class SaltTestsuiteParser(SaltCoverageTestingParser):
     source_code_basedir = SALT_ROOT
 
     def _get_suites(self, include_unit=False, include_cloud_provider=False,
-                    include_proxy=False):
+                    include_proxy=False, include_kitchen=False):
         '''
         Return a set of all test suites except unit and cloud provider tests
         unless requested
@@ -191,28 +200,34 @@ class SaltTestsuiteParser(SaltCoverageTestingParser):
             suites -= set(['cloud_provider'])
         if not include_proxy:
             suites -= set(['proxy'])
+        if not include_kitchen:
+            suites -= set(['kitchen'])
 
         return suites
 
     def _check_enabled_suites(self, include_unit=False,
-                              include_cloud_provider=False, include_proxy=False):
+                              include_cloud_provider=False,
+                              include_proxy=False,
+                              include_kitchen=False):
         '''
         Query whether test suites have been enabled
         '''
         suites = self._get_suites(include_unit=include_unit,
                                   include_cloud_provider=include_cloud_provider,
-                                  include_proxy=include_proxy)
+                                  include_proxy=include_proxy,
+                                  include_kitchen=include_kitchen)
 
         return any([getattr(self.options, suite) for suite in suites])
 
     def _enable_suites(self, include_unit=False, include_cloud_provider=False,
-                       include_proxy=False):
+                       include_proxy=False, include_kitchen=False):
         '''
         Enable test suites for current test run
         '''
         suites = self._get_suites(include_unit=include_unit,
                                   include_cloud_provider=include_cloud_provider,
-                                  include_proxy=include_proxy)
+                                  include_proxy=include_proxy,
+                                  include_kitchen=include_kitchen)
 
         for suite in suites:
             setattr(self.options, suite, True)
@@ -382,6 +397,15 @@ class SaltTestsuiteParser(SaltCoverageTestingParser):
             action='store_true',
             help='Run unit tests'
         )
+        self.test_selection_group.add_option(
+            '-k',
+            '--kitchen',
+            '--kitchen-tests',
+            dest='kitchen',
+            default=False,
+            action='store_true',
+            help='Run kitchen tests'
+        )
         self.test_selection_group.add_option(
             '--fileserver',
             '--fileserver-tests',
@@ -463,14 +487,31 @@ class SaltTestsuiteParser(SaltCoverageTestingParser):
             default=False,
             help='Run venafi runner tests'
         )
+        self.test_selection_group.add_option(
+            '--daemons',
+            '--daemon-tests',
+            dest='daemons',
+            action='store_true',
+            default=False,
+            help='Run salt/daemons/*.py tests'
+        )
+        self.test_selection_group.add_option(
+            '--scheduler',
+            dest='scheduler',
+            action='store_true',
+            default=False,
+            help='Run scheduler integration tests'
+        )
 
     def validate_options(self):
         if self.options.cloud_provider or self.options.external_api:
             # Turn on expensive tests execution
             os.environ['EXPENSIVE_TESTS'] = 'True'
 
-        import salt.utils
-        if salt.utils.is_windows():
+        # This fails even with salt.utils.platform imported in the global
+        # scope, unless we import it again here.
+        import salt.utils.platform
+        if salt.utils.platform.is_windows():
             import salt.utils.win_functions
             current_user = salt.utils.win_functions.get_current_user()
             if current_user == 'SYSTEM':
@@ -503,7 +544,8 @@ class SaltTestsuiteParser(SaltCoverageTestingParser):
         if not self.options.name and not \
                 self._check_enabled_suites(include_unit=True,
                                            include_cloud_provider=True,
-                                           include_proxy=True):
+                                           include_proxy=True,
+                                           include_kitchen=True):
             self._enable_suites(include_unit=True)
 
         self.start_coverage(
@@ -530,7 +572,7 @@ class SaltTestsuiteParser(SaltCoverageTestingParser):
         return self.run_suite(full_path, display_name, suffix='test_*.py')
 
     def start_daemons_only(self):
-        if not salt.utils.is_windows():
+        if not salt.utils.platform.is_windows():
             self.set_filehandle_limits('integration')
         try:
             print_header(
@@ -606,7 +648,7 @@ class SaltTestsuiteParser(SaltCoverageTestingParser):
         for integration tests or unit tests
         '''
         # Get current limits
-        if salt.utils.is_windows():
+        if salt.utils.platform.is_windows():
             import win32file
             prev_hard = win32file._getmaxstdio()
             prev_soft = 512
@@ -642,7 +684,7 @@ class SaltTestsuiteParser(SaltCoverageTestingParser):
                 '{0}, hard: {1}'.format(soft, hard)
             )
             try:
-                if salt.utils.is_windows():
+                if salt.utils.platform.is_windows():
                     hard = 2048 if hard > 2048 else hard
                     win32file._setmaxstdio(hard)
                 else:
@@ -667,19 +709,19 @@ class SaltTestsuiteParser(SaltCoverageTestingParser):
 
         if self.options.name:
             for test in self.options.name:
-                if test.startswith(('tests.unit.', 'unit.')):
+                if test.startswith(('tests.unit.', 'unit.', 'test.kitchen.', 'kitchen.')):
                     named_unit_test.append(test)
                     continue
                 named_tests.append(test)
 
-        if (self.options.unit or named_unit_test) and not named_tests and not \
+        if (self.options.unit or self.options.kitchen or named_unit_test) and not named_tests and not \
                 self._check_enabled_suites(include_cloud_provider=True):
             # We're either not running any integration test suites, or we're
             # only running unit tests by passing --unit or by passing only
             # `unit.<whatever>` to --name.  We don't need the tests daemon
             # running
             return [True]
-        if not salt.utils.is_windows():
+        if not salt.utils.platform.is_windows():
             self.set_filehandle_limits('integration')
 
         try:
@@ -758,8 +800,41 @@ class SaltTestsuiteParser(SaltCoverageTestingParser):
             status.append(results)
         return status
 
+    def run_kitchen_tests(self):
+        '''
+        Execute the kitchen tests
+        '''
+        named_kitchen_test = []
+        if self.options.name:
+            for test in self.options.name:
+                if not test.startswith(('tests.kitchen.', 'kitchen.')):
+                    continue
+                named_kitchen_test.append(test)
 
-def main():
+        if not self.options.kitchen and not named_kitchen_test:
+            # We are not explicitly running the unit tests and none of the
+            # names passed to --name is a unit test.
+            return [True]
+
+        status = []
+        if self.options.kitchen:
+            results = self.run_suite(
+                os.path.join(TEST_DIR, 'kitchen'), 'Kitchen', suffix='test_*.py'
+            )
+            status.append(results)
+            # We executed ALL unittests, we can skip running unittests by name
+            # below
+            return status
+
+        for name in named_kitchen_test:
+            results = self.run_suite(
+                os.path.join(TEST_DIR, 'kitchen'), name, suffix='test_*.py', load_from_name=True
+            )
+            status.append(results)
+        return status
+
+
+def main(**kwargs):
     '''
     Parse command line options for running specific tests
     '''
@@ -771,6 +846,12 @@ def main():
         )
         parser.parse_args()
 
+        # Override parser options (helpful when importing runtests.py and
+        # running from within a REPL). Using kwargs.items() to avoid importing
+        # six, as this feature will rarely be used.
+        for key, val in kwargs.items():
+            setattr(parser.options, key, val)
+
         overall_status = []
         if parser.options.interactive:
             parser.start_daemons_only()
@@ -778,6 +859,8 @@ def main():
         overall_status.extend(status)
         status = parser.run_unit_tests()
         overall_status.extend(status)
+        status = parser.run_kitchen_tests()
+        overall_status.extend(status)
         false_count = overall_status.count(False)
 
         if false_count > 0:
