@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 '''
-    :codeauthor: :email:`Mike Place <mp@saltstack.com>`
+    :codeauthor: Mike Place <mp@saltstack.com>
 '''
 
 # Import Python libs
@@ -12,7 +12,7 @@ import tempfile
 # Import Salt Testing libs
 from tests.integration import AdaptedConfigurationTestCaseMixin
 from tests.support.mixins import LoaderModuleMockMixin
-from tests.support.paths import FILES, TMP, TMP_STATE_TREE
+from tests.support.paths import BASE_FILES, TMP, TMP_STATE_TREE
 from tests.support.unit import TestCase, skipIf
 from tests.support.mock import patch, NO_MOCK, NO_MOCK_REASON
 
@@ -20,6 +20,7 @@ from tests.support.mock import patch, NO_MOCK, NO_MOCK_REASON
 import salt.fileserver.roots as roots
 import salt.fileclient
 import salt.utils.files
+import salt.utils.hashutils
 import salt.utils.platform
 
 try:
@@ -63,13 +64,11 @@ class RootsTest(TestCase, AdaptedConfigurationTestCaseMixin, LoaderModuleMockMix
         else:
             cls.test_symlink_list_file_roots = None
         cls.tmp_dir = tempfile.mkdtemp(dir=TMP)
-        full_path_to_file = os.path.join(FILES, 'file', 'base', 'testfile')
+        full_path_to_file = os.path.join(BASE_FILES, 'testfile')
         with salt.utils.files.fopen(full_path_to_file, 'rb') as s_fp:
             with salt.utils.files.fopen(os.path.join(cls.tmp_dir, 'testfile'), 'wb') as d_fp:
                 for line in s_fp:
-                    d_fp.write(
-                        line.rstrip(b'\n').rstrip(b'\r') + os.linesep.encode('utf-8')
-                    )
+                    d_fp.write(line)
 
     @classmethod
     def tearDownClass(cls):
@@ -95,7 +94,7 @@ class RootsTest(TestCase, AdaptedConfigurationTestCaseMixin, LoaderModuleMockMix
         ret = roots.find_file('testfile')
         self.assertEqual('testfile', ret['rel'])
 
-        full_path_to_file = os.path.join(FILES, 'file', 'base', 'testfile')
+        full_path_to_file = os.path.join(BASE_FILES, 'testfile')
         self.assertEqual(full_path_to_file, ret['path'])
 
     def test_serve_file(self):
@@ -108,34 +107,9 @@ class RootsTest(TestCase, AdaptedConfigurationTestCaseMixin, LoaderModuleMockMix
                    'rel': 'testfile'}
             ret = roots.serve_file(load, fnd)
 
-            data = 'Scene 24\n\n \n  OLD MAN:  Ah, hee he he ha!\n  ' \
-                   'ARTHUR:  And this enchanter of whom you speak, he ' \
-                   'has seen the grail?\n  OLD MAN:  Ha ha he he he ' \
-                   'he!\n  ARTHUR:  Where does he live?  Old man, where ' \
-                   'does he live?\n  OLD MAN:  He knows of a cave, a ' \
-                   'cave which no man has entered.\n  ARTHUR:  And the ' \
-                   'Grail... The Grail is there?\n  OLD MAN:  Very much ' \
-                   'danger, for beyond the cave lies the Gorge\n      ' \
-                   'of Eternal Peril, which no man has ever crossed.\n  ' \
-                   'ARTHUR:  But the Grail!  Where is the Grail!?\n  ' \
-                   'OLD MAN:  Seek you the Bridge of Death.\n  ARTHUR:  ' \
-                   'The Bridge of Death, which leads to the Grail?\n  ' \
-                   'OLD MAN:  Hee hee ha ha!\n\n'
-            if salt.utils.platform.is_windows():
-                data = 'Scene 24\r\n\r\n \r\n  OLD MAN:  Ah, hee he he ' \
-                       'ha!\r\n  ARTHUR:  And this enchanter of whom you ' \
-                       'speak, he has seen the grail?\r\n  OLD MAN:  Ha ha ' \
-                       'he he he he!\r\n  ARTHUR:  Where does he live?  Old ' \
-                       'man, where does he live?\r\n  OLD MAN:  He knows of ' \
-                       'a cave, a cave which no man has entered.\r\n  ' \
-                       'ARTHUR:  And the Grail... The Grail is there?\r\n  ' \
-                       'OLD MAN:  Very much danger, for beyond the cave lies ' \
-                       'the Gorge\r\n      of Eternal Peril, which no man ' \
-                       'has ever crossed.\r\n  ARTHUR:  But the Grail!  ' \
-                       'Where is the Grail!?\r\n  OLD MAN:  Seek you the ' \
-                       'Bridge of Death.\r\n  ARTHUR:  The Bridge of Death, ' \
-                       'which leads to the Grail?\r\n  OLD MAN:  Hee hee ha ' \
-                       'ha!\r\n\r\n'
+            with salt.utils.files.fopen(
+                    os.path.join(BASE_FILES, 'testfile'), 'rb') as fp_:
+                data = fp_.read()
 
             self.assertDictEqual(
                 ret,
@@ -163,9 +137,9 @@ class RootsTest(TestCase, AdaptedConfigurationTestCaseMixin, LoaderModuleMockMix
 
         # Hashes are different in Windows. May be how git translates line
         # endings
-        hsum = 'baba5791276eb99a7cc498fb1acfbc3b4bd96d24cfe984b4ed6b5be2418731df'
-        if salt.utils.platform.is_windows():
-            hsum = '754aa260e1f3e70f43aaf92149c7d1bad37f708c53304c37660e628d7553f687'
+        with salt.utils.files.fopen(
+                os.path.join(BASE_FILES, 'testfile'), 'rb') as fp_:
+            hsum = salt.utils.hashutils.sha256_digest(fp_.read())
 
         self.assertDictEqual(
             ret,
@@ -185,10 +159,15 @@ class RootsTest(TestCase, AdaptedConfigurationTestCaseMixin, LoaderModuleMockMix
         self.assertIn(UNICODE_DIRNAME, ret)
 
     def test_symlink_list(self):
-        if self.test_symlink_list_file_roots:
-            self.opts['file_roots'] = self.test_symlink_list_file_roots
-        ret = roots.symlink_list({'saltenv': 'base'})
-        self.assertDictEqual(ret, {'dest_sym': 'source_sym'})
+        orig_file_roots = self.opts['file_roots']
+        try:
+            if self.test_symlink_list_file_roots:
+                self.opts['file_roots'] = self.test_symlink_list_file_roots
+            ret = roots.symlink_list({'saltenv': 'base'})
+            self.assertDictEqual(ret, {'dest_sym': 'source_sym'})
+        finally:
+            if self.test_symlink_list_file_roots:
+                self.opts['file_roots'] = orig_file_roots
 
 
 class RootsLimitTraversalTest(TestCase, AdaptedConfigurationTestCaseMixin):
