@@ -17,7 +17,7 @@
     :codeauthor: Bo Maryniuk <bo@suse.de>
 '''
 # Import Python Libs
-from __future__ import absolute_import
+from __future__ import absolute_import, print_function, unicode_literals
 import os
 
 # Import Salt Testing Libs
