@@ -1,14 +1,13 @@
 # -*- coding: utf-8 -*-
 '''
-    :codeauthor: :email:`Jayesh Kariya <jayeshk@saltstack.com>`
+    :codeauthor: Jayesh Kariya <jayeshk@saltstack.com>
 '''
-
 # Import Python Libs
 from __future__ import absolute_import, unicode_literals, print_function
 
 # Import Salt Testing Libs
 from tests.support.mixins import LoaderModuleMockMixin
-from tests.support.unit import TestCase
+from tests.support.unit import TestCase, skipIf
 from tests.support.mock import (
     MagicMock,
     patch
@@ -18,6 +17,7 @@ from tests.support.mock import (
 import salt.modules.win_timezone as win_timezone
 
 
+@skipIf(not win_timezone.HAS_PYTZ, 'This test requires pytz')
 class WinTimezoneTestCase(TestCase, LoaderModuleMockMixin):
     '''
     Test cases for salt.modules.win_timezone
@@ -31,12 +31,13 @@ class WinTimezoneTestCase(TestCase, LoaderModuleMockMixin):
         '''
         Test if it get current timezone (i.e. Asia/Calcutta)
         '''
-        mock_cmd = MagicMock(side_effect=['India Standard Time',
-                                          'Indian Standard Time'])
-        with patch.dict(win_timezone.__salt__, {'cmd.run': mock_cmd}):
+        mock_read = MagicMock(side_effect=[{'vdata': 'India Standard Time'},
+                                           {'vdata': 'Indian Standard Time'}])
+
+        with patch.dict(win_timezone.__utils__, {'reg.read_value': mock_read}):
             self.assertEqual(win_timezone.get_zone(), 'Asia/Calcutta')
 
-            self.assertFalse(win_timezone.get_zone())
+            self.assertEqual(win_timezone.get_zone(), 'Unknown')
 
     # 'get_offset' function tests: 1
 
@@ -44,15 +45,15 @@ class WinTimezoneTestCase(TestCase, LoaderModuleMockMixin):
         '''
         Test if it get current numeric timezone offset from UCT (i.e. +0530)
         '''
-        time = ('(UTC+05:30) Chennai, Kolkata, Mumbai, \
-        New Delhi\nIndia Standard Time')
-        mock_cmd = MagicMock(side_effect=['India Standard Time', time])
-        with patch.dict(win_timezone.__salt__, {'cmd.run': mock_cmd}):
-            self.assertEqual(win_timezone.get_offset(), '+0530')
+        # time = ('(UTC+05:30) Chennai, Kolkata, Mumbai, \
+        # New Delhi\nIndia Standard Time')
+        # mock_cmd = MagicMock(side_effect=['India Standard Time', time])
+        # with patch.dict(win_timezone.__salt__, {'cmd.run': mock_cmd}):
 
-        mock_cmd = MagicMock(return_value='India Standard Time')
-        with patch.dict(win_timezone.__salt__, {'cmd.run': mock_cmd}):
-            self.assertFalse(win_timezone.get_offset())
+        mock_read = MagicMock(return_value={'vdata': 'India Standard Time'})
+
+        with patch.dict(win_timezone.__utils__, {'reg.read_value': mock_read}):
+            self.assertEqual(win_timezone.get_offset(), '+0530')
 
     # 'get_zonecode' function tests: 1
 
@@ -60,7 +61,10 @@ class WinTimezoneTestCase(TestCase, LoaderModuleMockMixin):
         '''
         Test if it get current timezone (i.e. PST, MDT, etc)
         '''
-        self.assertFalse(win_timezone.get_zonecode())
+        mock_read = MagicMock(return_value={'vdata': 'India Standard Time'})
+
+        with patch.dict(win_timezone.__utils__, {'reg.read_value': mock_read}):
+            self.assertEqual(win_timezone.get_zonecode(), 'IST')
 
     # 'set_zone' function tests: 1
 
@@ -68,8 +72,15 @@ class WinTimezoneTestCase(TestCase, LoaderModuleMockMixin):
         '''
         Test if it unlinks, then symlinks /etc/localtime to the set timezone.
         '''
-        mock_cmd = MagicMock(return_value=0)
-        with patch.dict(win_timezone.__salt__, {'cmd.retcode': mock_cmd}):
+        mock_cmd = MagicMock(return_value={'pid': 78,
+                                           'retcode': 0,
+                                           'stderr': '',
+                                           'stdout': ''})
+        mock_read = MagicMock(return_value={'vdata': 'India Standard Time'})
+
+        with patch.dict(win_timezone.__salt__, {'cmd.run_all': mock_cmd}), \
+                patch.dict(win_timezone.__utils__, {'reg.read_value': mock_read}):
+
             self.assertTrue(win_timezone.set_zone('Asia/Calcutta'))
 
     # 'zone_compare' function tests: 1
@@ -80,8 +91,9 @@ class WinTimezoneTestCase(TestCase, LoaderModuleMockMixin):
         the one set in /etc/localtime. Returns True if they match,
         and False if not. Mostly useful for running state checks.
         '''
-        mock_cmd = MagicMock(return_value='India Standard Time')
-        with patch.dict(win_timezone.__salt__, {'cmd.run': mock_cmd}):
+        mock_read = MagicMock(return_value={'vdata': 'India Standard Time'})
+
+        with patch.dict(win_timezone.__utils__, {'reg.read_value': mock_read}):
             self.assertTrue(win_timezone.zone_compare('Asia/Calcutta'))
 
     # 'get_hwclock' function tests: 1
