@@ -36,6 +36,7 @@ import salt.serializers.yaml as yamlserializer
 import salt.serializers.json as jsonserializer
 import salt.serializers.python as pythonserializer
 from salt.exceptions import CommandExecutionError
+import salt.utils.win_functions
 
 
 @skipIf(NO_MOCK, NO_MOCK_REASON)
@@ -58,6 +59,15 @@ class TestFileState(TestCase, LoaderModuleMockMixin):
             }
         }
 
+    def tearDown(self):
+        remove_dir = '/tmp/etc'
+        if salt.utils.platform.is_windows():
+            remove_dir = 'c:\\tmp\\etc'
+        try:
+            salt.utils.files.rm_rf(remove_dir)
+        except OSError:
+            pass
+
     def test_serialize(self):
         def returner(contents, *args, **kwargs):
             returner.returned = contents
@@ -143,10 +153,15 @@ class TestFileState(TestCase, LoaderModuleMockMixin):
         else:
             group = 'saltstack'
 
-        ret = {'name': name,
-               'result': False,
-               'comment': '',
-               'changes': {}}
+        def return_val(kwargs):
+            val = {
+                'name': name,
+                'result': False,
+                'comment': '',
+                'changes': {},
+            }
+            val.update(kwargs)
+            return val
 
         mock_t = MagicMock(return_value=True)
         mock_f = MagicMock(return_value=False)
@@ -160,7 +175,7 @@ class TestFileState(TestCase, LoaderModuleMockMixin):
 
         with patch.dict(filestate.__salt__, {'config.manage_mode': mock_t}):
             comt = ('Must provide name to file.symlink')
-            ret.update({'comment': comt, 'name': ''})
+            ret = return_val({'comment': comt, 'name': ''})
             self.assertDictEqual(filestate.symlink('', target), ret)
 
         with patch.dict(filestate.__salt__, {'config.manage_mode': mock_t,
@@ -168,8 +183,12 @@ class TestFileState(TestCase, LoaderModuleMockMixin):
                                              'file.group_to_gid': mock_empty,
                                              'user.info': mock_empty,
                                              'user.current': mock_user}):
-            comt = ('User {0} does not exist. Group {1} does not exist.'.format(user, group))
-            ret.update({'comment': comt, 'name': name})
+            if salt.utils.platform.is_windows():
+                comt = ('User {0} does not exist'.format(user))
+                ret = return_val({'comment': comt, 'name': name})
+            else:
+                comt = ('User {0} does not exist. Group {1} does not exist.'.format(user, group))
+                ret = return_val({'comment': comt, 'name': name})
             self.assertDictEqual(filestate.symlink(name, target, user=user,
                                                    group=group), ret)
 
@@ -181,11 +200,22 @@ class TestFileState(TestCase, LoaderModuleMockMixin):
                                              'user.current': mock_user}):
             with patch.dict(filestate.__opts__, {'test': True}):
                 with patch.object(os.path, 'exists', mock_f):
-                    comt = ('Symlink {0} to {1}'
-                            ' is set for creation').format(name, target)
-                    ret.update({'comment': comt,
-                                'result': None,
-                                'pchanges': {'new': name}})
+                    if salt.utils.platform.is_windows():
+                        comt = ('User {0} does not exist'.format(user))
+                        ret = return_val(
+                            {
+                                'comment': comt,
+                                'result': False,
+                                'name': name,
+                                'changes': {}
+                            }
+                        )
+                    else:
+                        comt = ('Symlink {0} to {1}'
+                                ' is set for creation').format(name, target)
+                        ret = return_val({'comment': comt,
+                                    'result': None,
+                                    'pchanges': {'new': name}})
                     self.assertDictEqual(filestate.symlink(name, target,
                                                            user=user,
                                                            group=group), ret)
@@ -199,10 +229,21 @@ class TestFileState(TestCase, LoaderModuleMockMixin):
             with patch.dict(filestate.__opts__, {'test': False}):
                 with patch.object(os.path, 'isdir', mock_f):
                     with patch.object(os.path, 'exists', mock_f):
-                        comt = ('Directory {0} for symlink is not present').format(test_dir)
-                        ret.update({'comment': comt,
+                        if salt.utils.platform.is_windows():
+                            comt = 'User {0} does not exist'.format(user)
+                            ret = return_val(
+                                {
+                                    'comment': comt,
                                     'result': False,
-                                    'pchanges': {'new': name}})
+                                    'name': name,
+                                    'changes': {},
+                                }
+                            )
+                        else:
+                            comt = ('Directory {0} for symlink is not present').format(test_dir)
+                            ret = return_val({'comment': comt,
+                                        'result': False,
+                                        'pchanges': {'new': name}})
                         self.assertDictEqual(filestate.symlink(name, target,
                                                                user=user,
                                                                group=group), ret)
@@ -217,15 +258,19 @@ class TestFileState(TestCase, LoaderModuleMockMixin):
             with patch.dict(filestate.__opts__, {'test': False}):
                 with patch.object(os.path, 'isdir', mock_t):
                     with patch.object(salt.states.file, '_check_symlink_ownership', mock_t):
-                        comt = ('Symlink {0} is present and owned by '
-                                '{1}:{2}'.format(name, user, group))
-                        ret.update({'comment': comt,
-                                    'result': True,
-                                    'pchanges': {}})
-                        self.assertDictEqual(filestate.symlink(name, target,
-                                                               user=user,
-                                                               group=group), ret)
-
+                        with patch('salt.utils.win_functions.get_sid_from_name', return_value='test-sid'):
+                            if salt.utils.platform.is_windows():
+                                comt = ('Symlink {0} is present and owned by '
+                                        '{1}'.format(name, user))
+                            else:
+                                comt = ('Symlink {0} is present and owned by '
+                                        '{1}:{2}'.format(name, user, group))
+                            ret = return_val({'comment': comt,
+                                        'result': True,
+                                        'pchanges': {}})
+                            self.assertDictEqual(filestate.symlink(name, target,
+                                                                   user=user,
+                                                                   group=group), ret)
         with patch.dict(filestate.__salt__, {'config.manage_mode': mock_t,
                                              'file.user_to_uid': mock_uid,
                                              'file.group_to_gid': mock_gid,
@@ -237,15 +282,16 @@ class TestFileState(TestCase, LoaderModuleMockMixin):
                 with patch.object(os.path, 'isdir', mock_t):
                     with patch.object(os.path, 'exists', mock_f):
                         with patch.object(os.path, 'lexists', mock_t):
-                            comt = ('File exists where the backup target SALT'
-                                    ' should go')
-                            ret.update({'comment': comt,
-                                        'result': False,
-                                        'pchanges': {'new': name}})
-                            self.assertDictEqual(filestate.symlink
-                                                 (name, target, user=user,
-                                                  group=group, backupname='SALT'),
-                                                 ret)
+                            with patch('salt.utils.win_functions.get_sid_from_name', return_value='test-sid'):
+                                comt = ('File exists where the backup target SALT'
+                                        ' should go')
+                                ret = return_val({'comment': comt,
+                                            'result': False,
+                                            'pchanges': {'new': name}})
+                                self.assertDictEqual(filestate.symlink
+                                                     (name, target, user=user,
+                                                      group=group, backupname='SALT'),
+                                                     ret)
 
         with patch.dict(filestate.__salt__, {'config.manage_mode': mock_t,
                                              'file.user_to_uid': mock_uid,
@@ -258,14 +304,15 @@ class TestFileState(TestCase, LoaderModuleMockMixin):
                 with patch.object(os.path, 'isdir', mock_t):
                     with patch.object(os.path, 'exists', mock_f):
                         with patch.object(os.path, 'isfile', mock_t):
-                            comt = ('File exists where the symlink {0} should be'
-                                    .format(name))
-                            ret.update({'comment': comt,
-                                        'pchanges': {'new': name},
-                                        'result': False})
-                            self.assertDictEqual(filestate.symlink
-                                                 (name, target, user=user,
-                                                  group=group), ret)
+                            with patch('salt.utils.win_functions.get_sid_from_name', return_value='test-sid'):
+                                comt = ('File exists where the symlink {0} should be'
+                                        .format(name))
+                                ret = return_val({'comment': comt,
+                                            'pchanges': {'new': name},
+                                            'result': False})
+                                self.assertDictEqual(filestate.symlink
+                                                     (name, target, user=user,
+                                                      group=group), ret)
 
         with patch.dict(filestate.__salt__, {'config.manage_mode': mock_t,
                                              'file.user_to_uid': mock_uid,
@@ -279,11 +326,12 @@ class TestFileState(TestCase, LoaderModuleMockMixin):
                 with patch.object(os.path, 'isdir', MagicMock(side_effect=[True, False])):
                     with patch.object(os.path, 'isfile', mock_t):
                         with patch.object(os.path, 'exists', mock_f):
-                            comt = ('File exists where the symlink {0} should be'.format(name))
-                            ret.update({'comment': comt, 'result': False})
-                            self.assertDictEqual(filestate.symlink
-                                                 (name, target, user=user,
-                                                  group=group), ret)
+                            with patch('salt.utils.win_functions.get_sid_from_name', return_value='test-sid'):
+                                comt = ('File exists where the symlink {0} should be'.format(name))
+                                ret = return_val({'comment': comt, 'result': False, 'pchanges': {'new': '/tmp/testfile.txt'}})
+                                self.assertDictEqual(filestate.symlink
+                                                     (name, target, user=user,
+                                                      group=group), ret)
 
         with patch.dict(filestate.__salt__, {'config.manage_mode': mock_t,
                                              'file.user_to_uid': mock_uid,
@@ -297,11 +345,12 @@ class TestFileState(TestCase, LoaderModuleMockMixin):
                 with patch.object(os.path, 'isdir', MagicMock(side_effect=[True, False])):
                     with patch.object(os.path, 'isdir', mock_t):
                         with patch.object(os.path, 'exists', mock_f):
-                            comt = ('Directory exists where the symlink {0} should be'.format(name))
-                            ret.update({'comment': comt, 'result': False})
-                            self.assertDictEqual(filestate.symlink
-                                                 (name, target, user=user,
-                                                  group=group), ret)
+                            with patch('salt.utils.win_functions.get_sid_from_name', return_value='test-sid'):
+                                comt = ('Directory exists where the symlink {0} should be'.format(name))
+                                ret = return_val({'comment': comt, 'result': False, 'pchanges': {'new': '/tmp/testfile.txt'}})
+                                self.assertDictEqual(filestate.symlink
+                                                     (name, target, user=user,
+                                                      group=group), ret)
 
         with patch.dict(filestate.__salt__, {'config.manage_mode': mock_t,
                                              'file.user_to_uid': mock_uid,
@@ -314,12 +363,13 @@ class TestFileState(TestCase, LoaderModuleMockMixin):
             with patch.dict(filestate.__opts__, {'test': False}):
                 with patch.object(os.path, 'isdir', MagicMock(side_effect=[True, False])):
                     with patch.object(os.path, 'isfile', mock_f):
-                        comt = ('Unable to create new symlink {0} -> '
-                                '{1}: '.format(name, target))
-                        ret.update({'comment': comt, 'result': False})
-                        self.assertDictEqual(filestate.symlink
-                                             (name, target, user=user,
-                                              group=group), ret)
+                        with patch('salt.utils.win_functions.get_sid_from_name', return_value='test-sid'):
+                            comt = ('Unable to create new symlink {0} -> '
+                                    '{1}: '.format(name, target))
+                            ret = return_val({'comment': comt, 'result': False, 'pchanges': {'new': '/tmp/testfile.txt'}})
+                            self.assertDictEqual(filestate.symlink
+                                                 (name, target, user=user,
+                                                  group=group), ret)
 
         with patch.dict(filestate.__salt__, {'config.manage_mode': mock_t,
                                              'file.user_to_uid': mock_uid,
@@ -334,13 +384,15 @@ class TestFileState(TestCase, LoaderModuleMockMixin):
             with patch.dict(filestate.__opts__, {'test': False}):
                 with patch.object(os.path, 'isdir', MagicMock(side_effect=[True, False])):
                     with patch.object(os.path, 'isfile', mock_f):
-                        comt = 'Created new symlink {0} -> {1}'.format(name, target)
-                        ret.update({'comment': comt,
-                                    'result': True,
-                                    'changes': {'new': name}})
-                        self.assertDictEqual(filestate.symlink
-                                             (name, target, user=user,
-                                              group=group), ret)
+                        with patch('salt.states.file._check_symlink_ownership', return_value=True):
+                            with patch('salt.utils.win_functions.get_sid_from_name', return_value='test-sid'):
+                                comt = 'Created new symlink {0} -> {1}'.format(name, target)
+                                ret = return_val({'comment': comt,
+                                            'result': True, 'pchanges': {'new': '/tmp/testfile.txt'},
+                                            'changes': {'new': name}})
+                                self.assertDictEqual(filestate.symlink
+                                                     (name, target, user=user,
+                                                      group=group), ret)
 
         with patch.dict(filestate.__salt__, {'config.manage_mode': mock_t,
                                              'file.user_to_uid': mock_uid,
@@ -355,15 +407,19 @@ class TestFileState(TestCase, LoaderModuleMockMixin):
             with patch.dict(filestate.__opts__, {'test': False}):
                 with patch.object(os.path, 'isdir', MagicMock(side_effect=[True, False])):
                     with patch.object(os.path, 'isfile', mock_f):
-                        comt = ('Created new symlink {0} -> {1}, '
-                                'but was unable to set ownership to '
-                                '{2}:{3}'.format(name, target, user, group))
-                        ret.update({'comment': comt,
-                                    'result': False,
-                                    'changes': {'new': name}})
-                        self.assertDictEqual(filestate.symlink
-                                             (name, target, user=user,
-                                              group=group), ret)
+                        with patch('salt.utils.win_functions.get_sid_from_name', return_value='test-sid'):
+                            with patch('salt.states.file._set_symlink_ownership', return_value=False):
+                                with patch('salt.states.file._check_symlink_ownership', return_value=False):
+                                    comt = ('Created new symlink {0} -> {1}, '
+                                            'but was unable to set ownership to '
+                                            '{2}:{3}'.format(name, target, user, group))
+                                    ret = return_val({'comment': comt,
+                                                'result': False,
+                                                'pchanges': {'new': '/tmp/testfile.txt'},
+                                                'changes': {'new': name}})
+                                    self.assertDictEqual(filestate.symlink
+                                                         (name, target, user=user,
+                                                          group=group), ret)
 
     # 'absent' function tests: 1
     def test_absent(self):
@@ -711,6 +767,8 @@ class TestFileState(TestCase, LoaderModuleMockMixin):
         name = '/etc/testdir'
         user = 'salt'
         group = 'saltstack'
+        if salt.utils.platform.is_windows():
+            name = name.replace('/', '\\')
 
         ret = {'name': name,
                'result': False,
@@ -807,7 +865,7 @@ class TestFileState(TestCase, LoaderModuleMockMixin):
                         else:
                             comt = ('The following files will be changed:\n{0}:'
                                     ' directory - new\n'.format(name))
-                        p_chg = {'/etc/testdir': {'directory': 'new'}}
+                        p_chg = {name: {'directory': 'new'}}
                         ret.update({
                             'comment': comt,
                             'result': None,
@@ -864,6 +922,8 @@ class TestFileState(TestCase, LoaderModuleMockMixin):
         source = 'salt://code/flask'
         user = 'salt'
         group = 'saltstack'
+        if salt.utils.platform.is_windows():
+            name = name.replace('/', '\\')
 
         ret = {'name': name,
                'result': False,
@@ -1130,7 +1190,10 @@ class TestFileState(TestCase, LoaderModuleMockMixin):
         '''
         Test to ensure that some text appears at the beginning of a file.
         '''
-        name = '/etc/motd'
+        name = '/tmp/etc/motd'
+        if salt.utils.platform.is_windows():
+            name = 'c:\\tmp\\etc\\motd'
+        assert not os.path.exists(os.path.split(name)[0])
         source = ['salt://motd/hr-messages.tmpl']
         sources = ['salt://motd/devops-messages.tmpl']
         text = ['Trust no one unless you have eaten much salt with him.']
@@ -1159,12 +1222,15 @@ class TestFileState(TestCase, LoaderModuleMockMixin):
                          'cp.get_template': mock_f,
                          'file.search': mock_f,
                          'file.prepend': mock_t}):
-            with patch.object(os.path, 'isdir', mock_t):
-                comt = ('The following files will be changed:\n/etc:'
-                        ' directory - new\n')
-                ret.update({'comment': comt, 'name': name, 'pchanges': {'/etc': {'directory': 'new'}}})
-                self.assertDictEqual(filestate.prepend(name, makedirs=True),
-                                     ret)
+            comt = ('The following files will be changed:\n/tmp/etc:'
+                    ' directory - new\n')
+            pchanges = {'/tmp/etc': {'directory': 'new'}}
+            if salt.utils.platform.is_windows():
+                comt = 'The directory "c:\\tmp\\etc" will be changed'
+                pchanges = {'c:\\tmp\\etc': {'directory': 'new'}}
+            ret.update({'comment': comt, 'name': name, 'pchanges': pchanges})
+            self.assertDictEqual(filestate.prepend(name, makedirs=True),
+                                 ret)
 
             with patch.object(os.path, 'isabs', mock_f):
                 comt = ('Specified file {0} is not an absolute path'
