@@ -15,7 +15,7 @@ from tests.support.unit import TestCase
 # Import Salt libs
 import tests.integration as integration
 import salt.modules.cmdmod
-import salt.utils
+import salt.utils.platform
 
 
 class DocTestCase(TestCase):
@@ -36,7 +36,7 @@ class DocTestCase(TestCase):
         '''
         salt_dir = integration.CODE_DIR
 
-        if salt.utils.is_windows():
+        if salt.utils.platform.is_windows():
             # No grep in Windows, use findstr
             # findstr in windows doesn't prepend 'Binary` to binary files, so
             # use the '/P' switch to skip files with unprintable characters
