@@ -1,8 +1,7 @@
 # -*- coding: utf-8 -*-
 '''
-    :codeauthor: :email:`Pedro Algarvio (pedro@algarvio.me)`
-    :codeauthor: :email:`Alexandru Bleotu (alexandru.bleotu@morganstanley.com)`
-
+    :codeauthor: Pedro Algarvio (pedro@algarvio.me)
+    :codeauthor: Alexandru Bleotu (alexandru.bleotu@morganstanley.com)
 
     tests.unit.pillar_test
     ~~~~~~~~~~~~~~~~~~~~~~
@@ -36,6 +35,7 @@ class PillarTestCase(TestCase):
     def test_pillarenv_from_saltenv(self):
         with patch('salt.pillar.compile_template') as compile_template:
             opts = {
+                'optimization_order': [0, 1, 2],
                 'renderer': 'json',
                 'renderer_blacklist': [],
                 'renderer_whitelist': [],
@@ -60,6 +60,7 @@ class PillarTestCase(TestCase):
 
     def test_ext_pillar_no_extra_minion_data_val_dict(self):
         opts = {
+            'optimization_order': [0, 1, 2],
             'renderer': 'json',
             'renderer_blacklist': [],
             'renderer_whitelist': [],
@@ -100,6 +101,7 @@ class PillarTestCase(TestCase):
 
     def test_ext_pillar_no_extra_minion_data_val_list(self):
         opts = {
+            'optimization_order': [0, 1, 2],
             'renderer': 'json',
             'renderer_blacklist': [],
             'renderer_whitelist': [],
@@ -140,6 +142,7 @@ class PillarTestCase(TestCase):
 
     def test_ext_pillar_no_extra_minion_data_val_elem(self):
         opts = {
+            'optimization_order': [0, 1, 2],
             'renderer': 'json',
             'renderer_blacklist': [],
             'renderer_whitelist': [],
@@ -178,6 +181,7 @@ class PillarTestCase(TestCase):
 
     def test_ext_pillar_with_extra_minion_data_val_dict(self):
         opts = {
+            'optimization_order': [0, 1, 2],
             'renderer': 'json',
             'renderer_blacklist': [],
             'renderer_whitelist': [],
@@ -218,6 +222,7 @@ class PillarTestCase(TestCase):
 
     def test_ext_pillar_with_extra_minion_data_val_list(self):
         opts = {
+            'optimization_order': [0, 1, 2],
             'renderer': 'json',
             'renderer_blacklist': [],
             'renderer_whitelist': [],
@@ -258,6 +263,7 @@ class PillarTestCase(TestCase):
 
     def test_ext_pillar_with_extra_minion_data_val_elem(self):
         opts = {
+            'optimization_order': [0, 1, 2],
             'renderer': 'json',
             'renderer_blacklist': [],
             'renderer_whitelist': [],
@@ -298,6 +304,7 @@ class PillarTestCase(TestCase):
 
     def test_dynamic_pillarenv(self):
         opts = {
+            'optimization_order': [0, 1, 2],
             'renderer': 'json',
             'renderer_blacklist': [],
             'renderer_whitelist': [],
@@ -312,6 +319,7 @@ class PillarTestCase(TestCase):
 
     def test_ignored_dynamic_pillarenv(self):
         opts = {
+            'optimization_order': [0, 1, 2],
             'renderer': 'json',
             'renderer_blacklist': [],
             'renderer_whitelist': [],
@@ -326,6 +334,7 @@ class PillarTestCase(TestCase):
     def test_malformed_pillar_sls(self):
         with patch('salt.pillar.compile_template') as compile_template:
             opts = {
+                'optimization_order': [0, 1, 2],
                 'renderer': 'json',
                 'renderer_blacklist': [],
                 'renderer_whitelist': [],
@@ -435,6 +444,7 @@ class PillarTestCase(TestCase):
 
     def test_includes_override_sls(self):
         opts = {
+            'optimization_order': [0, 1, 2],
             'renderer': 'json',
             'renderer_blacklist': [],
             'renderer_whitelist': [],
@@ -497,6 +507,7 @@ class PillarTestCase(TestCase):
         with patch('salt.pillar.salt.fileclient.get_file_client', autospec=True) as get_file_client, \
                 patch('salt.pillar.salt.minion.Matcher') as Matcher:  # autospec=True disabled due to py3 mock bug
             opts = {
+                'optimization_order': [0, 1, 2],
                 'renderer': 'yaml',
                 'renderer_blacklist': [],
                 'renderer_whitelist': [],
